==> sems-0.8.12.tar.gz: sems-0.8.12/AmAudio.h <==
/*
 * $Id: AmAudio.h,v 1.9.4.3 2005/08/25 07:54:04 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _AmAudio_h_
#define _AmAudio_h_

#include "AmThread.h"
#include "amci/amci.h"

#include <stdio.h>

#include <memory>
using std::auto_ptr;
#include <string>
using std::string;

/**
 * Implements double buffering.
 */

class DblBuffer
{
    /** Buffer. */
    unsigned char samples[AUDIO_BUFFER_SIZE * 2];
    /** 0 for first buffer, 1 for the second. */
    int active_buf;

public:
    /** Constructs a double buffer. */
    DblBuffer();
    /** Returns a pointer to the current front buffer. */
    operator unsigned char*();
    /** Returns a pointer to the current back buffer. */
    unsigned char* back_buffer();
    /** swaps front and back buffer. */
    void swap();
};

struct amci_codec_t;
struct amci_inoutfmt_t;
struct amci_file_desc_t;
struct amci_subtype_t;

class AmAudio;

/**
 * Audio format structure.
 * Holds a description of the format.
 * @todo Create two child class:
 * <ul>
 *   <li>file based format
 *   <li>payload based format
 * </ul>
 */

class AmAudioFormat
{
public:
    /** Sampling size (in bytes). */
    int sample;
    /** Number of channels. */
    int channels;
    /** Sampling rate. */
    int rate;

    AmAudioFormat();
    virtual ~AmAudioFormat();

    /** @return The format's codec pointer. */
    amci_codec_t*    getCodec();
    /** @return Handler returned by the codec's init function.*/
    long getHCodec();

    /** @return true if same format. */
    bool operator == (const AmAudioFormat& r) const;
    /** @return false if same format. */
    bool operator != (const AmAudioFormat& r) const;

protected:
    virtual int getCodecId()=0;

    /** ==0 if not yet initialized. */
    amci_codec_t*   codec;
    /** ==0 if not yet initialized. */
    long             h_codec;

    /** Calls amci_codec_t::destroy() */
    void destroyCodec();
    /** Calls amci_codec_t::init() */
    void initCodec();

private:
    void operator = (const AmAudioFormat& r);
};

class AmAudioSimpleFormat: public AmAudioFormat
{
    int codec_id;

protected:
    int getCodecId() { return codec_id; }

public:
    AmAudioSimpleFormat(int codec_id);
};

class AmAudioFileFormat: public AmAudioFormat
{
    /** == "" if not yet initialized. */
    string          name;
    
    /** == -1 if not yet initialized. */
    int             subtype;

    /** ==  0 if not yet initialized. */
    amci_subtype_t* p_subtype;

protected:
    int getCodecId();

public:
    /**
     * Constructor for file based formats.
     * The subtype will be the default one.
     * If called without parameter, the resulting 
     * audio format structure will not be valid. 
     * Just for initialization in AmAudio::AmAudio().
     * @param name The file format name (ex. "Wav")
     */
    // AmAudioFormat(const string& name = "");
    /**
     * Constructor for file based formats.
     * All information are taken from the plug-in description.
     * @param name The file format name (ex. "Wav").
     * @param subtype Subtype for the file format (see amci.h).
     */
    AmAudioFileFormat(const string& name, int subtype = -1);
    /**
     * Constructor for file based formats.
     * All information are taken from the file descriptor.
     * @param name The file format name (ex. "Wav").
     * @param fd A file descriptor filled by the a plug-in's open function.
     */
    AmAudioFileFormat(const string& name, const amci_file_desc_t* fd);

    /** @return Format name. */
    string        getName() { return name; }
    /** @return Format subtype. */
    int           getSubtypeId() { return subtype; }
    /** @return Subtype pointer. */
    amci_subtype_t*  getSubtype();

    void setSubtypeId(int subtype_id) 
    { 
	destroyCodec();
	subtype = subtype_id; 
	p_subtype = 0;
	codec = getCodec();
    }
};

class AmAudioRtpFormat: public AmAudioFormat
{
    /** ==-1 if not yet initialized. */
    int     payload;

protected:
    int getCodecId();

public:
    /**
     * Constructor for payload based formats.
     * All the information are taken from the 
     * payload description in the originating plug-in.
     */
    AmAudioRtpFormat(int payload);

    /** @return Payload ID. */
    int           getPayload() { return payload; }
};

/**
 * AmAudio binds an input and an output format together.
 * It converts also the samples if needed, 
 * depending on the input/output formats.<br>
 * <br>Internal Format: PCM signed 16 bit (mono | stereo).
 */

class AmAudio
{
protected:
    /** Sample buffer. */
    DblBuffer samples;

    /** Size of datas having been read/written until now. */
    int data_size;

    /** Gets 'size' bytes directly from stream (Read,Pull). */
    virtual int streamGet(unsigned int user_ts, unsigned int size) = 0;
    /** Puts 'size' bytes directly from stream (Write,Push). */
    virtual int streamPut(unsigned int user_ts, unsigned int size) = 0;

    /** 
     * Converts a buffer from mono to stereo. 
     * @param size [in,out] size in bytes
     * <ul><li>before: size = input size</li><li>after: size = output size</li></ul>
     */
    void mono2stereo(unsigned char* out_buf,unsigned char* in_buf,int& size);
    /** 
     * Converts a buffer from stereo to mono. 
     * @param size [in,out] size in bytes
     * <ul><li>Before call is size = input size</li><li>After the call is size = output size</li></ul>
     */
    void stereo2mono(unsigned char* out_buf,unsigned char* in_buf,int& size);

    /** 
     * Converts sound from in format to out format. 
     *
     * @warning For packet based payloads / file formats, use:
     * <pre>           nb_sample = input buffer size / sample size of the reference format
     * </pre>           whereby the format with/from which the codec works is the reference one.
     */
    int convert(unsigned int user_ts, unsigned char* buffer,int nb_samples, bool is_get);

    /**
     * Converts from the input format to the internal format.
     * <ul><li>input = front buffer</li><li>output = back buffer</li></ul>
     * @param codec see AmAudioFormat::getCodec()
     * @param h_codec see AmAudioFormat::getHCodec()
     * <br><br>
     * @param size [in,out] size in bytes
     * <ul><li>before: size = input size</li><li>after: size = output size</li></ul>
     */
    int convertInFmt2Pcm16(amci_codec_t* codec,long h_codec,int& size);
    /**
     * Converts from the input format to the internal format.
     * <ul><li>input = front buffer</li><li>output = back buffer</li></ul>
     * @param codec see AmAudioFormat::getCodec()
     * @param h_codec see AmAudioFormat::getHCodec()
     * <br><br>
     * @param size [in,out] size in bytes
     * <ul><li>before: size = input size</li><li>after: size = output size</li></ul>
     */
    int convertPcm162OutFmt(amci_codec_t* codec,long h_codec,int& size);
    int convertChannels(int& size);

public:
    bool begin_talk;

    /** Input format. @see AmAudioFormat */
    auto_ptr<AmAudioFormat> in;
    /** Output format. @see AmAudioFormat */
    auto_ptr<AmAudioFormat> out;

    AmAudio();
    virtual ~AmAudio();

    /** Gets data size in the current file */
    int getDataSize() { return data_size; }

    /** Closes the audio pipe. */
    virtual void close();

    /** 
     * Get some samples from input stream.
     * @warning For packet based payloads / file formats, use:
     * <pre>           nb_sample = input buffer size / sample size of the reference format
     * </pre>           whereby the format with/from which the codec works is the reference one.
     * @return # bytes read, else -1 if error (0 is OK) 
     */
    virtual int get(unsigned int user_ts, unsigned char* buffer, unsigned int nb_samples);

    /** 
     * Put some samples to the output stream.
     * @warning For packet based payloads / file formats, use:
     * <pre>           nb_sample = input buffer size / sample size of the reference format
     * </pre>           whereby the format with/from which the codec works is the reference one.
     * @return # bytes written, else -1 if error (0 is OK) 
     */
    virtual int put(unsigned int user_ts, unsigned char* buffer, unsigned int nb_samples);
};


class AmAudioFile: public AmAudio
{
public:
    /** Open mode. */
    enum OpenMode { Read=1, Write=2 };

protected:
    /** Pointer to the file opened as last. */
    FILE* fp;
    long begin;

    /** Format of that file. @see fp, open(). */
    amci_inoutfmt_t* iofmt;
    /** Open mode. */
    int open_mode;

    /** @see AmAudio::streamGet */
    int streamGet(unsigned int user_ts, unsigned int size);

    /** @see AmAudio::streamPut */
    int streamPut(unsigned int user_ts, unsigned int size);

public:
    AmSharedVar<bool> loop;

    AmAudioFile(const string& file_fmt, int subtype=-1);
    ~AmAudioFile();

    /**
     * Opens a file.
     * <ul>
     * <li>In read mode: sets input format.
     * <li>In write mode: <ol>
     *                    <li>needs output format set. 
     *                    <li>If file name already exists, 
     *                        the file will be overwritten.
     *                    </ol>
     * </ul>
     * @param filename Name of the file.
     * @param mode Open mode.
     * @return 0 if everything's OK
     * @see OpenMode
     */
    int open(const char* filename, OpenMode mode);

    /** Closes the file. */
    void close();
};

#endif

// Local Variables:
// mode:C++
// End:

==> sems-0.9.0.tar.gz: sems-0.9.0/AmAudio.h <==
/*
 * $Id: AmAudio.h,v 1.16.4.1 2005/08/25 09:34:20 rco Exp $
 *
 * Copyright (C) 2002-2003 Fhg Fokus
 *
 * This file is part of sems, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the ser software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * sems is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _AmAudio_h_
#define _AmAudio_h_

#include "AmThread.h"
#include "amci/amci.h"

#include <stdio.h>

#include <memory>
using std::auto_ptr;
#include <string>
using std::string;

#define AUDIO_LOCK_FMT


/**
 * Implements double buffering.
 */

class DblBuffer
{
    /** Buffer. */
    unsigned char samples[AUDIO_BUFFER_SIZE * 2];
    /** 0 for first buffer, 1 for the second. */
    int active_buf;

public:
    /** Constructs a double buffer. */
    DblBuffer();
    /** Returns a pointer to the current front buffer. */
    operator unsigned char*();
    /** Returns a pointer to the current back buffer. */
    unsigned char* back_buffer();
    /** swaps front and back buffer. */
    void swap();
};

struct amci_codec_t;
struct amci_inoutfmt_t;
struct amci_file_desc_t;
struct amci_subtype_t;

class AmAudio;

/**
 * Audio format structure.
 * Holds a description of the format.
 * @todo Create two child class:
 * <ul>
 *   <li>file based format
 *   <li>payload based format
 * </ul>
 */

class AmAudioFormat
{
public:
    /** Sampling size (in bytes). */
    int sample;
    /** Number of channels. */
    int channels;
    /** Sampling rate. */
    int rate;
  /* frame length in samples (frame based codecs) */
  int frame_length;
  /* frame size in bytes */
  int frame_size;
  /* encoded frame size in bytes */
  int frame_encoded_size;

  string sdp_format_parameters;

    AmAudioFormat();
    virtual ~AmAudioFormat();

    /** @return The format's codec pointer. */
    amci_codec_t*    getCodec();
    /** @return Handler returned by the codec's init function.*/
    long             getHCodec();
    long             getHCodecNoInit() { return h_codec; } // do not initialize

    /** @return true if same format. */
    bool operator == (const AmAudioFormat& r) const;
    /** @return false if same format. */
    bool operator != (const AmAudioFormat& r) const;

protected:
    virtual int getCodecId()=0;

    /** ==0 if not yet initialized. */
    amci_codec_t*   codec;
    /** ==0 if not yet initialized. */
    long            h_codec;

    /** Calls amci_codec_t::destroy() */
    void destroyCodec();
    /** Calls amci_codec_t::init() */
    void initCodec();

private:
    void operator = (const AmAudioFormat& r);
};

class AmAudioSimpleFormat: public AmAudioFormat
{
    int codec_id;

protected:
    int getCodecId() { return codec_id; }

public:
    AmAudioSimpleFormat(int codec_id);
};

class AmAudioFileFormat: public AmAudioFormat
{
    /** == "" if not yet initialized. */
    string          name;
    
    /** == -1 if not yet initialized. */
    int             subtype;

    /** ==  0 if not yet initialized. */
    amci_subtype_t* p_subtype;

protected:
    int getCodecId();

public:
    /**
     * Constructor for file based formats.
     * The subtype will be the default one.
     * If called without parameter, the resulting 
     * audio format structure will not be valid. 
     * Just for initialization in AmAudio::AmAudio().
     * @param name The file format name (ex. "Wav")
     */
    // AmAudioFormat(const string& name = "");
    /**
     * Constructor for file based formats.
     * All information are taken from the plug-in description.
     * @param name The file format name (ex. "Wav").
     * @param subtype Subtype for the file format (see amci.h).
     */
    AmAudioFileFormat(const string& name, int subtype = -1);
    /**
     * Constructor for file based formats.
     * All information are taken from the file descriptor.
     * @param name The file format name (ex. "Wav").
     * @param fd A file descriptor filled by the a plug-in's open function.
     */
    AmAudioFileFormat(const string& name, const amci_file_desc_t* fd);

    /** @return Format name. */
    string        getName() { return name; }
    /** @return Format subtype. */
    int           getSubtypeId() { return subtype; }
    /** @return Subtype pointer. */
    amci_subtype_t*  getSubtype();

    void setSubtypeId(int subtype_id) 
    { 
	destroyCodec();
	subtype = subtype_id; 
	p_subtype = 0;
	codec = getCodec();
    }
};

class AmAudioRtpFormat: public AmAudioFormat
{
  /** ==-1 if not yet initialized. */
  int             payload;
  /** == 0 if not yet initialized. */
  amci_payload_t* amci_pl;

protected:
    int getCodecId();
    amci_payload_t* getPayloadP();

public:
    /**
     * Constructor for payload based formats.
     * All the information are taken from the 
     * payload description in the originating plug-in.
     */
    AmAudioRtpFormat(int payload, string format_parameters);

    /** @return Payload ID. */
    int           getPayload() { return payload; }
};

/**
 * AmAudio binds an input and an output format together.
 * It converts also the samples if needed, 
 * depending on the input/output formats.<br>
 * <br>Internal Format: PCM signed 16 bit (mono | stereo).
 */

class AmAudio
{
private:
  AmMutex inFmtLock;
  AmMutex outFmtLock;
protected:
    /** Sample buffer. */
    DblBuffer samples;

    /** Size of datas having been read/written until now. */
    int data_size;

    /** Gets 'size' bytes directly from stream (Read,Pull). */
    virtual int streamGet(unsigned int user_ts, unsigned int size) = 0;
    /** Puts 'size' bytes directly from stream (Write,Push). */
    virtual int streamPut(unsigned int user_ts, unsigned int size) = 0;

    /** 
     * Converts a buffer from mono to stereo. 
     * @param size [in,out] size in bytes
     * <ul><li>before: size = input size</li><li>after: size = output size</li></ul>
     */
    void mono2stereo(unsigned char* out_buf,unsigned char* in_buf,int& size);
    /** 
     * Converts a buffer from stereo to mono. 
     * @param size [in,out] size in bytes
     * <ul><li>Before call is size = input size</li><li>After the call is size = output size</li></ul>
     */
    void stereo2mono(unsigned char* out_buf,unsigned char* in_buf,int& size);

    /** 
     * Converts sound from in format to out format. 
     *
     * @warning For packet based payloads / file formats, use:
     * <pre>           nb_sample = input buffer size / sample size of the reference format
     * </pre>           whereby the format with/from which the codec works is the reference one.
     */
    int convert(unsigned int user_ts, unsigned char* buffer,int nb_samples, bool is_get);

    /**
     * Converts from the input format to the internal format.
     * <ul><li>input = front buffer</li><li>output = back buffer</li></ul>
     * @param codec see AmAudioFormat::getCodec()
     * @param h_codec see AmAudioFormat::getHCodec()
     * <br><br>
     * @param size [in,out] size in bytes
     * <ul><li>before: size = input size</li><li>after: size = output size</li></ul>
     */
    int convertInFmt2Pcm16(amci_codec_t* codec,long h_codec,int& size);
    /**
     * Converts from the input format to the internal format.
     * <ul><li>input = front buffer</li><li>output = back buffer</li></ul>
     * @param codec see AmAudioFormat::getCodec()
     * @param h_codec see AmAudioFormat::getHCodec()
     * <br><br>
     * @param size [in,out] size in bytes
     * <ul><li>before: size = input size</li><li>after: size = output size</li></ul>
     */
    int convertPcm162OutFmt(amci_codec_t* codec,long h_codec,int& size);
    int convertChannels(int& size);

public:
    bool begin_talk;

    /** Input format. @see AmAudioFormat */
    auto_ptr<AmAudioFormat> in;
    /** Output format. @see AmAudioFormat */
    auto_ptr<AmAudioFormat> out;

    AmAudio();
    virtual ~AmAudio();

    /** Gets data size in the current file */
    int getDataSize() { return data_size; }

    /** Closes the audio pipe. */
    virtual void close();

    /** 
     * Get some samples from input stream.
     * @warning For packet based payloads / file formats, use:
     * <pre>           nb_sample = input buffer size / sample size of the reference format
     * </pre>           whereby the format with/from which the codec works is the reference one.
     * @return # bytes read, else -1 if error (0 is OK) 
     */
    virtual int get(unsigned int user_ts, unsigned char* buffer, unsigned int nb_samples);

    /** 
     * Put some samples to the output stream.
     * @warning For packet based payloads / file formats, use:
     * <pre>           nb_sample = input buffer size / sample size of the reference format
     * </pre>           whereby the format with/from which the codec works is the reference one.
     * @return # bytes written, else -1 if error (0 is OK) 
     */
    virtual int put(unsigned int user_ts, unsigned char* buffer, unsigned int nb_samples);
  
#ifdef AUDIO_LOCK_FMT
  void lockInFmt() { inFmtLock.lock(); }
  void unlockInFmt() { inFmtLock.unlock(); }
  void lockOutFmt() { outFmtLock.lock(); }
  void unlockOutFmt() { outFmtLock.unlock(); }
  void lockInOutFmts() { lockInFmt(); lockOutFmt(); }
  void unlockInOutFmts() { unlockInFmt(); unlockOutFmt(); }
#else
  void lockInFmt() {}
  void unlockInFmt() {}
  void lockOutFmt() {}
  void unlockOutFmt() {}
  void lockInOutFmts() {}
  void unlockInOutFmts() {}
#endif
};


class AmAudioFile: public AmAudio
{
public:
    /** Open mode. */
    enum OpenMode { Read=1, Write=2 };

protected:
    /** Pointer to the file opened as last. */
    FILE* fp;
    long begin;

    /** Format of that file. @see fp, open(). */
    amci_inoutfmt_t* iofmt;
    /** Open mode. */
    int open_mode;

    /** @see AmAudio::streamGet */
    int streamGet(unsigned int user_ts, unsigned int size);

    /** @see AmAudio::streamPut */
    int streamPut(unsigned int user_ts, unsigned int size);

public:
    AmSharedVar<bool> loop;

    AmAudioFile(const string& file_fmt, int subtype=-1);
    ~AmAudioFile();

    /**
     * Opens a file.
     * <ul>
     * <li>In read mode: sets input format.
     * <li>In write mode: <ol>
     *                    <li>needs output format set. 
     *                    <li>If file name already exists, 
     *                        the file will be overwritten.
     *                    </ol>
     * </ul>
     * @param filename Name of the file.
     * @param mode Open mode.
     * @return 0 if everything's OK
     * @see OpenMode
     */
    int open(const char* filename, OpenMode mode);

    /** Closes the file. */
    void close();

    /**
     * @return MIME type corresponding to the audio file.
     */
    string getMimeType();
};

#endif

// Local Variables:
// mode:C++
// End:
